"""
Task Queue

Manages task execution pipeline with status tracking.

Mirrors: src/core/contract-ai/evolution/task-queue.ts
@version 1.0.0
"""

from datetime import datetime
from enum import Enum
from typing import Literal, Optional

from pydantic import BaseModel, Field


# ============================================================================
# TYPES
# ============================================================================

TaskStatus = Literal["pending", "running", "done", "failed", "skipped"]


class Task(BaseModel):
    """Task definition"""
    id: str
    name: str
    status: TaskStatus = "pending"
    started_at: Optional[datetime] = Field(default=None, alias="startedAt")
    completed_at: Optional[datetime] = Field(default=None, alias="completedAt")
    error: Optional[str] = None
    children: Optional[list["Task"]] = None
    
    model_config = {"populate_by_name": True}


# ============================================================================
# TASK QUEUE CLASS
# ============================================================================

class TaskQueue:
    """
    Task queue with status tracking and YAML output.
    
    Example:
        queue = TaskQueue(verbose=True)
        task = queue.add("Generate code")
        queue.start(task.id)
        # ... do work ...
        queue.done(task.id)
    """
    
    def __init__(self, verbose: bool = True):
        self.tasks: list[Task] = []
        self.verbose = verbose
        self._renderer: Optional["ShellRenderer"] = None
    
    @property
    def renderer(self) -> "ShellRenderer":
        """Lazy load renderer"""
        if self._renderer is None:
            from .shell_renderer import ShellRenderer
            self._renderer = ShellRenderer(self.verbose)
        return self._renderer
    
    def add(self, name: str, task_id: Optional[str] = None) -> Task:
        """Add a new task to the queue"""
        task = Task(
            id=task_id or f"task-{len(self.tasks) + 1}",
            name=name,
            status="pending"
        )
        self.tasks.append(task)
        return task
    
    def start(self, task_id: str) -> None:
        """Mark a task as running"""
        task = self._find_task(task_id)
        if task:
            task.status = "running"
            task.started_at = datetime.now()
            self._print_todo_list()
    
    def done(self, task_id: str) -> None:
        """Mark a task as done"""
        task = self._find_task(task_id)
        if task:
            task.status = "done"
            task.completed_at = datetime.now()
            
            all_done = all(
                t.status in ("done", "failed", "skipped")
                for t in self.tasks
            )
            if all_done:
                self._print_todo_list()
    
    def fail(self, task_id: str, error: str) -> None:
        """Mark a task as failed"""
        task = self._find_task(task_id)
        if task:
            task.status = "failed"
            task.error = error
            task.completed_at = datetime.now()
            self._print_todo_list(force=True)
    
    def skip(self, task_id: str) -> None:
        """Mark a task as skipped"""
        task = self._find_task(task_id)
        if task:
            task.status = "skipped"
            task.completed_at = datetime.now()
    
    def get_task(self, task_id: str) -> Optional[Task]:
        """Get task by ID"""
        return self._find_task(task_id)
    
    def get_stats(self) -> dict[str, int]:
        """Get task statistics"""
        return {
            "pending": len([t for t in self.tasks if t.status == "pending"]),
            "running": len([t for t in self.tasks if t.status == "running"]),
            "done": len([t for t in self.tasks if t.status == "done"]),
            "failed": len([t for t in self.tasks if t.status == "failed"]),
            "skipped": len([t for t in self.tasks if t.status == "skipped"]),
            "total": len(self.tasks),
        }
    
    def print(self, force: bool = True) -> None:
        """Print current task list"""
        self._print_todo_list(force=force)
    
    def _find_task(self, task_id: str) -> Optional[Task]:
        """Find task by ID"""
        return next((t for t in self.tasks if t.id == task_id), None)
    
    def _print_todo_list(self, force: bool = False) -> None:
        """Print task list as YAML"""
        if not self.verbose:
            return
        
        stats = self.get_stats()
        all_done = stats["pending"] == 0 and stats["running"] == 0
        just_started = stats["done"] == 0 and stats["running"] == 1
        
        if just_started or force:
            self.renderer.heading(2, "TODO")
            yaml_lines = [
                "# @type: task_queue",
                "# @description: Evolution pipeline task list",
                "progress:",
                f"  done: {stats['done']}",
                f"  total: {stats['total']}",
            ]
            if stats["failed"] > 0:
                yaml_lines.append(f"  failed: {stats['failed']}")
            
            yaml_lines.append("tasks:")
            for task in self.tasks:
                error_line = ""
                if task.status == "failed" and task.error:
                    escaped_error = task.error[:180].replace('"', '\\"')
                    error_line = f'\n    error: "{escaped_error}"'
                yaml_lines.append(f'  - name: "{task.name}"\n    status: {task.status}{error_line}')
            
            self.renderer.codeblock("yaml", "\n".join(yaml_lines))
            
        elif all_done:
            self.renderer.heading(2, "COMPLETED")
            yaml_lines = [
                "# @type: task_queue_result",
                "# @description: Final task execution results",
                "progress:",
                f"  done: {stats['done']}",
                f"  total: {stats['total']}",
            ]
            if stats["failed"] > 0:
                yaml_lines.append(f"  failed: {stats['failed']}")
            
            yaml_lines.append("tasks:")
            for task in self.tasks:
                duration = 0
                if task.started_at and task.completed_at:
                    duration = int((task.completed_at - task.started_at).total_seconds())
                
                error_line = ""
                if task.status == "failed" and task.error:
                    escaped_error = task.error[:180].replace('"', '\\"')
                    error_line = f'\n    error: "{escaped_error}"'
                
                yaml_lines.append(
                    f'  - name: "{task.name}"\n    status: {task.status}\n    duration_sec: {duration}{error_line}'
                )
            
            self.renderer.codeblock("yaml", "\n".join(yaml_lines))
    
    @staticmethod
    def get_icon(status: TaskStatus) -> str:
        """Get status icon"""
        icons = {
            "pending": "⏳",
            "running": "🔄",
            "done": "✅",
            "failed": "❌",
            "skipped": "⏭️",
        }
        return icons.get(status, "•")



"""
Reclapp Models - Pydantic models for Contract AI

Replaces TypeScript types from src/core/contract-ai/types/
"""

from .definition import (
    FieldType,
    BasicFieldType,
    ExtendedFieldType,
    RelationType,
    FieldAnnotations,
    FieldDefinition,
    RelationDefinition,
    IndexDefinition,
    EntityDefinition,
    EventDefinition,
    WorkflowStepType,
    WorkflowStepDefinition,
    WorkflowDefinition,
    ApiOperation,
    ApiResourceDefinition,
    ApiEndpointDefinition,
    ApiDefinition,
    AppDefinition,
    DefinitionLayer,
)

from .generation import (
    InstructionPriority,
    Instruction,
    PatternType,
    Pattern,
    ConstraintType,
    Constraint,
    BackendTechStack,
    FrontendTechStack,
    DatabaseTechStack,
    TechStack,
    GenerationLayer,
)

from .validation import (
    AssertionType,
    Assertion,
    TestType,
    TestDefinition,
    StaticRule,
    QualityGate,
    AcceptanceCriteria,
    ValidationLayer,
)

from .contract import (
    ContractMetadata,
    ContractAI,
    is_valid_contract,
)

__all__ = [
    # Definition types
    "FieldType",
    "BasicFieldType", 
    "ExtendedFieldType",
    "RelationType",
    "FieldAnnotations",
    "FieldDefinition",
    "RelationDefinition",
    "IndexDefinition",
    "EntityDefinition",
    "EventDefinition",
    "WorkflowStepType",
    "WorkflowStepDefinition",
    "WorkflowDefinition",
    "ApiOperation",
    "ApiResourceDefinition",
    "ApiEndpointDefinition",
    "ApiDefinition",
    "AppDefinition",
    "DefinitionLayer",
    # Generation types
    "InstructionPriority",
    "Instruction",
    "PatternType",
    "Pattern",
    "ConstraintType",
    "Constraint",
    "BackendTechStack",
    "FrontendTechStack",
    "DatabaseTechStack",
    "TechStack",
    "GenerationLayer",
    # Validation types
    "AssertionType",
    "Assertion",
    "TestType",
    "TestDefinition",
    "StaticRule",
    "QualityGate",
    "AcceptanceCriteria",
    "ValidationLayer",
    # Contract
    "ContractMetadata",
    "ContractAI",
    "is_valid_contract",
]



"""
Dynamic Task Executor for Python

Watches task files and executes commands with live markdown output.
Similar to Dockerfile-style execution with parallel workers.

Mirrors: src/core/contract-ai/evolution/task-executor.ts
@version 1.0.0
"""

import asyncio
import os
import re
import time
from datetime import datetime
from typing import Dict, List, Optional, Set, Any
from pathlib import Path

from .runner import ShellRenderer


class ExecutorTask:
    """Task definition for the executor"""
    def __init__(self, id: str, command: str, timeout: int = 60000, cwd: Optional[str] = None):
        self.id = id
        self.command = command
        self.timeout = timeout
        self.cwd = cwd or os.getcwd()
        self.state = "pending"  # pending, running, done, failed, timeout
        self.started_at: Optional[float] = None
        self.completed_at: Optional[float] = None
        self.exit_code: Optional[int] = None
        self.stdout: List[str] = []
        self.stderr: List[str] = []
        self.error: Optional[str] = None


class TaskExecutor:
    """
    Executes commands from a task file or manually added.
    Supports parallel execution with a fixed number of workers.
    """
    
    def __init__(self, max_workers: int = 3, verbose: bool = True):
        self.max_workers = max_workers
        self.verbose = verbose
        self.renderer = ShellRenderer(verbose=verbose)
        self.tasks: Dict[str, ExecutorTask] = {}
        self.queue: List[str] = []
        self.running: Set[str] = set()
        self.task_counter = 0
        self.loop = asyncio.get_event_loop()
        
    def add_task(self, command: str, timeout: int = 60000, task_id: Optional[str] = None) -> ExecutorTask:
        """Add a single task to the queue"""
        self.task_counter += 1
        tid = task_id or f"task-{self.task_counter}"
        task = ExecutorTask(tid, command, timeout=timeout)
        self.tasks[tid] = task
        self.queue.append(tid)
        return task
        
    def add_tasks_from_file(self, file_path: str) -> None:
        """Parse tasks from a file (Dockerfile-style or bash lines)"""
        p = Path(file_path)
        if not p.exists():
            self.renderer.error(f"Task file not found: {file_path}")
            return
            
        content = p.read_text()
        for line in content.splitlines():
            trimmed = line.strip()
            if not trimmed or trimmed.startswith("#"):
                continue
                
            # Parse command and optional timeout: cmd # timeout: 120
            timeout = 60000
            match = re.match(r"^(.+?)\s+#\s*timeout:\s*(\d+)$", trimmed)
            if match:
                cmd_part = match.group(1)
                timeout = int(match.group(2)) * 1000
            else:
                cmd_part = trimmed
                
            self.add_task(cmd_part, timeout=timeout)
            
    async def run_task(self, task_id: str) -> None:
        """Execute a single task using subprocess"""
        task = self.tasks[task_id]
        task.state = "running"
        task.started_at = time.time()
        self.running.add(task_id)
        
        self.renderer.info(f"🚀 [START] {task.command}")
        
        try:
            # Create subprocess
            proc = await asyncio.create_subprocess_shell(
                task.command,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
                cwd=task.cwd
            )
            
            # Read output concurrently with timeout
            async def read_stream(stream, is_stderr=False):
                while True:
                    line = await stream.readline()
                    if not line:
                        break
                    decoded = line.decode().strip()
                    if is_stderr:
                        task.stderr.append(decoded)
                        self.renderer.text(f"   ⚠️ {decoded}")
                    else:
                        task.stdout.append(decoded)
                        self.renderer.text(f"   {decoded}")

            try:
                # Wait for process with timeout
                await asyncio.wait_for(
                    asyncio.gather(
                        proc.wait(),
                        read_stream(proc.stdout),
                        read_stream(proc.stderr)
                    ),
                    timeout=task.timeout / 1000.0
                )
                task.exit_code = proc.returncode
                task.state = "done" if task.exit_code == 0 else "failed"
            except asyncio.TimeoutError:
                proc.kill()
                task.state = "timeout"
                task.error = f"Timeout after {task.timeout}ms"
                self.renderer.error(f"⏱️ [TIMEOUT] {task.command}")
                
        except Exception as e:
            task.state = "failed"
            task.error = str(e)
            self.renderer.error(f"❌ [ERROR] {task.command}: {e}")
            
        finally:
            task.completed_at = time.time()
            self.running.remove(task_id)
            duration = int(task.completed_at - task.started_at)
            icon = "✅" if task.state == "done" else "❌"
            if task.state == "timeout": icon = "⏱️"
            self.renderer.info(f"{icon} [{task.state.upper()}] {task.command} ({duration}s)")
            
    async def wait_for_all(self) -> None:
        """Run tasks from queue until all finished using parallel workers"""
        pending_futures = []
        
        while self.queue or self.running:
            # Fill workers
            while len(self.running) < self.max_workers and self.queue:
                tid = self.queue.pop(0)
                future = asyncio.create_task(self.run_task(tid))
                pending_futures.append(future)
                
            if pending_futures:
                # Wait for at least one to complete
                done, pending = await asyncio.wait(
                    pending_futures, 
                    return_when=asyncio.FIRST_COMPLETED
                )
                pending_futures = list(pending)
            else:
                # Nothing running, nothing in queue
                break
                
    def print_summary(self) -> None:
        """Print final status of all tasks in YAML/Markdown"""
        tasks = list(self.tasks.values())
        done = len([t for t in tasks if t.state == "done"])
        failed = len([t for t in tasks if t.state in ("failed", "timeout")])
        
        self.renderer.heading(2, "Final Status")
        
        yaml_lines = [
            "tasks:",
            f"  total: {len(tasks)}",
            f"  done: {done}",
            f"  failed: {failed}",
            "",
            "results:"
        ]
        
        for t in tasks:
            duration = int(t.completed_at - t.started_at) if t.completed_at and t.started_at else 0
            icon = "✅" if t.state == "done" else "❌"
            if t.state == "timeout": icon = "⏱️"
            yaml_lines.append(f'  - {icon} "{t.command}": {t.state} ({duration}s)')
            if t.error:
                yaml_lines.append(f'      error: "{t.error}"')
                
        self.renderer.codeblock("yaml", "\n".join(yaml_lines))



"""
Evolution Generators

Template generators for Docker, Frontend, CI/CD, Database, and Documentation.

Extracted from evolution_manager.py to reduce file size.
@version 2.0.0
"""

import json
from pathlib import Path
from typing import Any, Optional


class EvolutionGenerators:
    """
    Template generators for evolution pipeline.
    
    Generates additional files like Docker, Frontend, CI/CD, etc.
    """
    
    def __init__(self, contract: Optional[dict] = None, port: int = 3000):
        self._contract = contract
        self._port = port
    
    def set_contract(self, contract: dict) -> None:
        """Set contract for generation"""
        self._contract = contract
    
    async def generate_database(self, target_dir: str) -> None:
        """Generate Prisma database schema"""
        if not self._contract:
            return
        
        entities = self._contract.get("definition", {}).get("entities", [])
        
        models = ""
        for entity in entities:
            name = entity.get("name", "Item")
            fields = entity.get("fields", [])
            
            field_defs = []
            for field in fields:
                fname = field.get("name", "")
                ftype = field.get("type", "String")
                
                prisma_type = {
                    "UUID": "String @id @default(uuid())",
                    "String": "String",
                    "Int": "Int",
                    "Float": "Float",
                    "Boolean": "Boolean",
                    "DateTime": "DateTime @default(now())",
                    "Date": "DateTime",
                }.get(ftype, "String")
                
                if fname == "id":
                    field_defs.append(f"  {fname} {prisma_type}")
                elif "At" in fname:
                    field_defs.append(f"  {fname} DateTime @default(now())")
                else:
                    field_defs.append(f"  {fname} {prisma_type}")
            
            models += f"""
model {name} {{
{chr(10).join(field_defs)}
}}
"""
        
        prisma_schema = f'''// Prisma Schema
// Generated by Reclapp Evolution Engine

generator client {{
  provider = "prisma-client-js"
}}

datasource db {{
  provider = "sqlite"
  url      = env("DATABASE_URL")
}}
{models}
'''
        
        prisma_dir = Path(target_dir) / "api" / "prisma"
        prisma_dir.mkdir(parents=True, exist_ok=True)
        
        schema_path = prisma_dir / "schema.prisma"
        schema_path.write_text(prisma_schema)
        
        env_content = 'DATABASE_URL="file:./dev.db"\n'
        env_path = Path(target_dir) / "api" / ".env"
        env_path.write_text(env_content)
    
    async def generate_frontend(self, target_dir: str) -> None:
        """Generate React frontend"""
        if not self._contract:
            return
        
        entities = self._contract.get("definition", {}).get("entities", [])
        app_name = self._contract.get("definition", {}).get("app", {}).get("name", "App")
        
        frontend_dir = Path(target_dir) / "frontend"
        src_dir = frontend_dir / "src"
        src_dir.mkdir(parents=True, exist_ok=True)
        
        # Package.json
        package_json = {
            "name": f"{app_name.lower().replace(' ', '-')}-frontend",
            "version": "1.0.0",
            "type": "module",
            "scripts": {
                "dev": "vite",
                "build": "vite build",
                "preview": "vite preview"
            },
            "dependencies": {
                "react": "^18.2.0",
                "react-dom": "^18.2.0"
            },
            "devDependencies": {
                "@types/react": "^18.2.0",
                "@types/react-dom": "^18.2.0",
                "@vitejs/plugin-react": "^4.2.0",
                "typescript": "^5.3.0",
                "vite": "^5.0.0"
            }
        }
        (frontend_dir / "package.json").write_text(json.dumps(package_json, indent=2))
        
        # Vite config
        vite_config = '''import { defineConfig } from 'vite'
import react from '@vitejs/plugin-react'

export default defineConfig({
  plugins: [react()],
  server: {
    port: 5173,
    proxy: {
      '/api': 'http://localhost:3000'
    }
  }
})
'''
        (frontend_dir / "vite.config.ts").write_text(vite_config)
        
        # Index.html
        index_html = f'''<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <title>{app_name}</title>
</head>
<body>
  <div id="root"></div>
  <script type="module" src="/src/main.tsx"></script>
</body>
</html>
'''
        (frontend_dir / "index.html").write_text(index_html)
        
        # Main.tsx
        main_tsx = '''import React from 'react'
import ReactDOM from 'react-dom/client'
import App from './App'
import './index.css'

ReactDOM.createRoot(document.getElementById('root')!).render(
  <React.StrictMode>
    <App />
  </React.StrictMode>,
)
'''
        (src_dir / "main.tsx").write_text(main_tsx)
        
        entity_name = entities[0].get("name", "Item") if entities else "Item"
        entity_lower = entity_name.lower()
        
        # App.tsx
        app_tsx = f'''import {{ useState, useEffect }} from 'react'

interface {entity_name} {{
  id: string
  title?: string
  name?: string
  createdAt: string
}}

function App() {{
  const [items, setItems] = useState<{entity_name}[]>([])
  const [newItem, setNewItem] = useState('')
  const [loading, setLoading] = useState(true)

  useEffect(() => {{
    fetch('/api/v1/{entity_lower}s')
      .then(res => res.json())
      .then(data => {{
        setItems(Array.isArray(data) ? data : [])
        setLoading(false)
      }})
      .catch(() => setLoading(false))
  }}, [])

  const addItem = async () => {{
    if (!newItem.trim()) return
    const res = await fetch('/api/v1/{entity_lower}s', {{
      method: 'POST',
      headers: {{ 'Content-Type': 'application/json' }},
      body: JSON.stringify({{ title: newItem }})
    }})
    if (res.ok) {{
      const item = await res.json()
      setItems([...items, item])
      setNewItem('')
    }}
  }}

  const deleteItem = async (id: string) => {{
    await fetch(`/api/v1/{entity_lower}s/${{id}}`, {{ method: 'DELETE' }})
    setItems(items.filter(i => i.id !== id))
  }}

  return (
    <div style={{ padding: '2rem', fontFamily: 'system-ui' }}>
      <h1>{app_name}</h1>
      
      <div style={{ marginBottom: '1rem' }}>
        <input
          value={{newItem}}
          onChange={{e => setNewItem(e.target.value)}}
          placeholder="Add new item..."
          style={{ padding: '0.5rem', marginRight: '0.5rem', width: '300px' }}
        />
        <button onClick={{addItem}} style={{ padding: '0.5rem 1rem' }}>Add</button>
      </div>

      {{loading ? (
        <p>Loading...</p>
      ) : (
        <ul style={{ listStyle: 'none', padding: 0 }}>
          {{items.map(item => (
            <li key={{item.id}} style={{
              display: 'flex',
              justifyContent: 'space-between',
              padding: '0.5rem',
              borderBottom: '1px solid #eee',
              maxWidth: '400px'
            }}>
              <span>{{item.title || item.name || item.id}}</span>
              <button onClick={{() => deleteItem(item.id)}} style={{ color: 'red' }}>×</button>
            </li>
          ))}}
        </ul>
      )}}
    </div>
  )
}}

export default App
'''
        (src_dir / "App.tsx").write_text(app_tsx)
        
        # Index.css
        index_css = '''* {
  box-sizing: border-box;
  margin: 0;
  padding: 0;
}

body {
  font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
  background: #f5f5f5;
  color: #333;
}

button {
  cursor: pointer;
  border: none;
  background: #007bff;
  color: white;
  border-radius: 4px;
}

button:hover {
  background: #0056b3;
}

input {
  border: 1px solid #ddd;
  border-radius: 4px;
}
'''
        (src_dir / "index.css").write_text(index_css)
        
        # TypeScript config
        tsconfig = {
            "compilerOptions": {
                "target": "ES2020",
                "useDefineForClassFields": True,
                "lib": ["ES2020", "DOM", "DOM.Iterable"],
                "module": "ESNext",
                "skipLibCheck": True,
                "moduleResolution": "bundler",
                "allowImportingTsExtensions": True,
                "resolveJsonModule": True,
                "isolatedModules": True,
                "noEmit": True,
                "jsx": "react-jsx",
                "strict": True
            },
            "include": ["src"]
        }
        (frontend_dir / "tsconfig.json").write_text(json.dumps(tsconfig, indent=2))
    
    async def generate_dockerfile(self, target_dir: str) -> None:
        """Generate Dockerfile for the API"""
        dockerfile_content = '''FROM node:20-alpine

WORKDIR /app

COPY api/package*.json ./
RUN npm ci --only=production

COPY api/src ./src
COPY api/tsconfig.json ./

RUN npm run build

EXPOSE 3000

CMD ["node", "dist/index.js"]
'''
        dockerfile_path = Path(target_dir) / "Dockerfile"
        dockerfile_path.write_text(dockerfile_content)
        
        compose_content = '''version: '3.8'

services:
  api:
    build: .
    ports:
      - "3000:3000"
    environment:
      - NODE_ENV=production
    restart: unless-stopped
'''
        compose_path = Path(target_dir) / "docker-compose.yml"
        compose_path.write_text(compose_content)
    
    async def generate_cicd(self, target_dir: str) -> None:
        """Generate CI/CD templates"""
        github_dir = Path(target_dir) / ".github" / "workflows"
        github_dir.mkdir(parents=True, exist_ok=True)
        
        ci_content = '''name: CI

on:
  push:
    branches: [main]
  pull_request:
    branches: [main]

jobs:
  build:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v4
      - uses: actions/setup-node@v4
        with:
          node-version: '20'
      - run: cd api && npm ci
      - run: cd api && npm run build
      - run: cd api && npm test
'''
        ci_path = github_dir / "ci.yml"
        ci_path.write_text(ci_content)
    
    async def generate_docs(self, target_dir: str) -> None:
        """Generate README documentation"""
        if not self._contract:
            return
        
        app_name = self._contract.get("definition", {}).get("app", {}).get("name", "App")
        entities = self._contract.get("definition", {}).get("entities", [])
        
        entity_docs = ""
        for entity in entities:
            name = entity.get("name", "Entity")
            fields = entity.get("fields", [])
            field_list = "\n".join([f"  - `{f.get('name')}`: {f.get('type')}" for f in fields])
            entity_docs += f"\n### {name}\n{field_list}\n"
        
        readme_content = f'''# {app_name}

Generated by Reclapp Evolution Engine.

## Quick Start

```bash
cd api
npm install
npm run dev
```

## API Endpoints

- `GET /health` - Health check
- `GET /api/v1/{{entity}}s` - List all
- `GET /api/v1/{{entity}}s/:id` - Get by ID
- `POST /api/v1/{{entity}}s` - Create
- `PUT /api/v1/{{entity}}s/:id` - Update
- `DELETE /api/v1/{{entity}}s/:id` - Delete

## Entities
{entity_docs}

## Docker

```bash
docker-compose up -d
```

## Development

Generated with ❤️ by Reclapp
'''
        readme_path = Path(target_dir) / "README.md"
        readme_path.write_text(readme_content)
    
    async def generate_test_fixtures(self, target_dir: str) -> None:
        """Generate test fixture files"""
        if not self._contract:
            return
        
        entities = self._contract.get("definition", {}).get("entities", [])
        fixtures_dir = Path(target_dir) / "tests" / "fixtures"
        fixtures_dir.mkdir(parents=True, exist_ok=True)
        
        for entity in entities:
            name = entity.get("name", "Item")
            name_lower = name.lower()
            
            fixture = {
                "valid": {
                    "title": f"Test {name}",
                    "description": f"A test {name_lower} for E2E testing",
                    "status": "pending"
                },
                "invalid": {
                    "description": "Missing required title field"
                },
                "update": {
                    "title": f"Updated {name}",
                    "description": "Updated description",
                    "status": "completed"
                }
            }
            
            fixture_path = fixtures_dir / f"{name_lower}.fixture.json"
            fixture_path.write_text(json.dumps(fixture, indent=2))
    
    async def generate_test_config(self, target_dir: str) -> None:
        """Generate test configuration file"""
        config = {
            "baseUrl": f"http://localhost:{self._port}",
            "apiPrefix": "/api/v1",
            "timeout": 5000,
            "retries": 3
        }
        
        config_path = Path(target_dir) / "tests" / "test.config.ts"
        config_content = f'''// Test Configuration
// Generated by Reclapp Evolution Engine

export const testConfig = {{
  baseUrl: '{config["baseUrl"]}',
  apiPrefix: '{config["apiPrefix"]}',
  timeout: {config["timeout"]},
  retries: {config["retries"]}
}};

export const getApiUrl = (path: string) => 
  `${{testConfig.baseUrl}}${{testConfig.apiPrefix}}${{path}}`;
'''
        config_path.write_text(config_content)
    
    async def generate_e2e_tests(self, target_dir: str) -> Optional[str]:
        """Generate E2E test file"""
        if not self._contract:
            return None
        
        entities = self._contract.get("definition", {}).get("entities", [])
        if not entities:
            return None
        
        entity = entities[0]
        entity_name = entity.get("name", "Item")
        entity_lower = entity_name.lower()
        
        test_content = f'''/**
 * E2E Tests for {entity_name} API
 * Auto-generated by Evolution Manager
 */

const BASE_URL = 'http://localhost:{self._port}';

interface TestResult {{
  name: string;
  passed: boolean;
  error?: string;
}}

async function runTests(): Promise<TestResult[]> {{
  const results: TestResult[] = [];
  let createdId: string | null = null;

  // Health check
  try {{
    const res = await fetch(`${{BASE_URL}}/health`);
    results.push({{ name: 'Health check', passed: res.ok }});
  }} catch (e) {{
    results.push({{ name: 'Health check', passed: false, error: String(e) }});
  }}

  // CREATE
  try {{
    const res = await fetch(`${{BASE_URL}}/api/v1/{entity_lower}s`, {{
      method: 'POST',
      headers: {{ 'Content-Type': 'application/json' }},
      body: JSON.stringify({{ title: 'Test {entity_name}' }})
    }});
    const data = await res.json();
    createdId = data.id;
    results.push({{ name: 'CREATE POST', passed: res.status === 201 && !!createdId }});
  }} catch (e) {{
    results.push({{ name: 'CREATE POST', passed: false, error: String(e) }});
  }}

  // READ ALL
  try {{
    const res = await fetch(`${{BASE_URL}}/api/v1/{entity_lower}s`);
    results.push({{ name: 'READ ALL', passed: res.ok }});
  }} catch (e) {{
    results.push({{ name: 'READ ALL', passed: false, error: String(e) }});
  }}

  // READ ONE
  if (createdId) {{
    try {{
      const res = await fetch(`${{BASE_URL}}/api/v1/{entity_lower}s/${{createdId}}`);
      results.push({{ name: 'READ ONE', passed: res.ok }});
    }} catch (e) {{
      results.push({{ name: 'READ ONE', passed: false, error: String(e) }});
    }}
  }}

  // UPDATE
  if (createdId) {{
    try {{
      const res = await fetch(`${{BASE_URL}}/api/v1/{entity_lower}s/${{createdId}}`, {{
        method: 'PUT',
        headers: {{ 'Content-Type': 'application/json' }},
        body: JSON.stringify({{ title: 'Updated {entity_name}' }})
      }});
      results.push({{ name: 'UPDATE PUT', passed: res.ok }});
    }} catch (e) {{
      results.push({{ name: 'UPDATE PUT', passed: false, error: String(e) }});
    }}
  }}

  // DELETE
  if (createdId) {{
    try {{
      const res = await fetch(`${{BASE_URL}}/api/v1/{entity_lower}s/${{createdId}}`, {{
        method: 'DELETE'
      }});
      results.push({{ name: 'DELETE', passed: res.status === 204 || res.ok }});
    }} catch (e) {{
      results.push({{ name: 'DELETE', passed: false, error: String(e) }});
    }}
  }}

  return results;
}}

runTests().then(results => {{
  console.log(JSON.stringify(results));
  const failed = results.filter(r => !r.passed).length;
  process.exit(failed > 0 ? 1 : 0);
}});
'''
        
        test_file = Path(target_dir) / "tests" / "e2e" / "api.e2e.ts"
        test_file.parent.mkdir(parents=True, exist_ok=True)
        test_file.write_text(test_content)
        return str(test_file)



"""
State Analyzer Module

Multi-level state analysis for evolution pipeline.
Tracks contract, code, and runtime state discrepancies.

Mirrors: src/core/contract-ai/evolution/state-analyzer.ts
"""

import json
from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Any, Optional


class StateLevel(str, Enum):
    """State analysis levels"""
    CONTRACT = "contract"
    CODE = "code"
    RUNTIME = "runtime"


class DiscrepancyType(str, Enum):
    """Types of state discrepancies"""
    MISSING_ENTITY = "missing_entity"
    MISSING_FIELD = "missing_field"
    TYPE_MISMATCH = "type_mismatch"
    MISSING_ENDPOINT = "missing_endpoint"
    MISSING_FILE = "missing_file"
    SCHEMA_MISMATCH = "schema_mismatch"
    RUNTIME_ERROR = "runtime_error"


@dataclass
class StateDiscrepancy:
    """A discrepancy between expected and actual state"""
    level: StateLevel
    type: DiscrepancyType
    path: str
    expected: Any
    actual: Any
    message: str
    severity: str = "error"
    
    def to_dict(self) -> dict:
        return {
            "level": self.level.value,
            "type": self.type.value,
            "path": self.path,
            "expected": self.expected,
            "actual": self.actual,
            "message": self.message,
            "severity": self.severity
        }


@dataclass
class ContractState:
    """State extracted from contract"""
    app_name: str = ""
    version: str = "1.0.0"
    entities: list[dict] = field(default_factory=list)
    endpoints: list[dict] = field(default_factory=list)
    tech_stack: dict = field(default_factory=dict)


@dataclass
class CodeState:
    """State extracted from generated code"""
    files: list[str] = field(default_factory=list)
    models: list[str] = field(default_factory=list)
    routes: list[str] = field(default_factory=list)
    has_package_json: bool = False
    has_tsconfig: bool = False


@dataclass
class RuntimeState:
    """State extracted from running service"""
    is_running: bool = False
    port: int = 0
    health_ok: bool = False
    endpoints_responding: list[str] = field(default_factory=list)


@dataclass
class MultiLevelState:
    """Combined multi-level state"""
    contract: ContractState = field(default_factory=ContractState)
    code: CodeState = field(default_factory=CodeState)
    runtime: RuntimeState = field(default_factory=RuntimeState)
    discrepancies: list[StateDiscrepancy] = field(default_factory=list)
    
    def is_consistent(self) -> bool:
        """Check if all levels are consistent"""
        return len([d for d in self.discrepancies if d.severity == "error"]) == 0
    
    def to_dict(self) -> dict:
        return {
            "contract": {
                "app_name": self.contract.app_name,
                "version": self.contract.version,
                "entities": len(self.contract.entities),
                "endpoints": len(self.contract.endpoints)
            },
            "code": {
                "files": len(self.code.files),
                "models": len(self.code.models),
                "routes": len(self.code.routes)
            },
            "runtime": {
                "is_running": self.runtime.is_running,
                "health_ok": self.runtime.health_ok
            },
            "discrepancies": [d.to_dict() for d in self.discrepancies],
            "is_consistent": self.is_consistent()
        }


class StateAnalyzer:
    """
    Analyzes state across contract, code, and runtime levels.
    
    Mirrors TypeScript StateAnalyzer for consistency checking.
    
    Example:
        analyzer = StateAnalyzer(target_dir)
        state = await analyzer.analyze(contract)
        
        if not state.is_consistent():
            for d in state.discrepancies:
                print(f"{d.level}: {d.message}")
    """
    
    def __init__(self, target_dir: str, verbose: bool = False):
        self.target_dir = Path(target_dir)
        self.verbose = verbose
    
    async def analyze(self, contract: Optional[dict] = None) -> MultiLevelState:
        """Perform full multi-level analysis"""
        state = MultiLevelState()
        
        # Analyze contract level
        if contract:
            state.contract = self._analyze_contract(contract)
        else:
            contract = self._load_contract()
            if contract:
                state.contract = self._analyze_contract(contract)
        
        # Analyze code level
        state.code = self._analyze_code()
        
        # Analyze runtime level
        state.runtime = await self._analyze_runtime()
        
        # Find discrepancies
        state.discrepancies = self._find_discrepancies(state)
        
        return state
    
    def _load_contract(self) -> Optional[dict]:
        """Load contract from file"""
        contract_path = self.target_dir / "contract" / "contract.ai.json"
        if contract_path.exists():
            try:
                return json.loads(contract_path.read_text())
            except:
                pass
        return None
    
    def _analyze_contract(self, contract: dict) -> ContractState:
        """Extract state from contract"""
        state = ContractState()
        
        definition = contract.get("definition", {})
        app = definition.get("app", {})
        
        state.app_name = app.get("name", "")
        state.version = app.get("version", "1.0.0")
        state.entities = definition.get("entities", [])
        
        # Extract endpoints from API definition
        api = definition.get("api", {})
        resources = api.get("resources", [])
        for resource in resources:
            if isinstance(resource, dict):
                state.endpoints.extend(resource.get("endpoints", []))
            elif isinstance(resource, str):
                # Generate CRUD endpoints for resource
                resource_lower = resource.lower()
                state.endpoints.extend([
                    {"method": "GET", "path": f"/api/{resource_lower}"},
                    {"method": "POST", "path": f"/api/{resource_lower}"},
                    {"method": "GET", "path": f"/api/{resource_lower}/:id"},
                    {"method": "PUT", "path": f"/api/{resource_lower}/:id"},
                    {"method": "DELETE", "path": f"/api/{resource_lower}/:id"},
                ])
        
        generation = contract.get("generation", {})
        state.tech_stack = generation.get("techStack", {})
        
        return state
    
    def _analyze_code(self) -> CodeState:
        """Extract state from generated code"""
        state = CodeState()
        
        # Find all files
        if self.target_dir.exists():
            for path in self.target_dir.rglob("*"):
                if path.is_file() and "node_modules" not in str(path):
                    rel_path = str(path.relative_to(self.target_dir))
                    state.files.append(rel_path)
                    
                    # Categorize files
                    if "/models/" in rel_path or rel_path.endswith(".model.ts"):
                        state.models.append(rel_path)
                    elif "/routes/" in rel_path or rel_path.endswith(".routes.ts"):
                        state.routes.append(rel_path)
        
        # Check for key files
        state.has_package_json = (self.target_dir / "api" / "package.json").exists() or \
                                  (self.target_dir / "package.json").exists()
        state.has_tsconfig = (self.target_dir / "api" / "tsconfig.json").exists() or \
                              (self.target_dir / "tsconfig.json").exists()
        
        return state
    
    async def _analyze_runtime(self) -> RuntimeState:
        """Extract state from running service"""
        import urllib.request
        import urllib.error
        
        state = RuntimeState()
        state.port = 3000  # Default port
        
        # Check health endpoint
        try:
            url = f"http://localhost:{state.port}/health"
            req = urllib.request.urlopen(url, timeout=2)
            if req.status == 200:
                state.is_running = True
                state.health_ok = True
        except:
            pass
        
        # Check API endpoints if service is running
        if state.is_running:
            test_endpoints = ["/api/v1", "/api"]
            for endpoint in test_endpoints:
                try:
                    url = f"http://localhost:{state.port}{endpoint}"
                    req = urllib.request.urlopen(url, timeout=2)
                    if req.status in [200, 404]:  # 404 means route exists but no data
                        state.endpoints_responding.append(endpoint)
                except:
                    pass
        
        return state
    
    def _find_discrepancies(self, state: MultiLevelState) -> list[StateDiscrepancy]:
        """Find discrepancies between state levels"""
        discrepancies = []
        
        # Check contract vs code
        for entity in state.contract.entities:
            entity_name = entity.get("name", "")
            if entity_name:
                # Check if model file exists
                model_found = any(
                    entity_name.lower() in f.lower() 
                    for f in state.code.models + state.code.files
                )
                if not model_found:
                    discrepancies.append(StateDiscrepancy(
                        level=StateLevel.CODE,
                        type=DiscrepancyType.MISSING_ENTITY,
                        path=f"models/{entity_name}",
                        expected=entity_name,
                        actual=None,
                        message=f"Entity '{entity_name}' defined in contract but no model file found",
                        severity="warning"
                    ))
        
        # Check required files
        if not state.code.has_package_json:
            discrepancies.append(StateDiscrepancy(
                level=StateLevel.CODE,
                type=DiscrepancyType.MISSING_FILE,
                path="package.json",
                expected="package.json",
                actual=None,
                message="Missing package.json file",
                severity="error"
            ))
        
        # Check runtime vs contract
        if state.contract.entities and not state.runtime.is_running:
            discrepancies.append(StateDiscrepancy(
                level=StateLevel.RUNTIME,
                type=DiscrepancyType.RUNTIME_ERROR,
                path="service",
                expected="running",
                actual="stopped",
                message="Service is not running",
                severity="warning"
            ))
        
        return discrepancies
    
    def write_state_snapshot(self, state: MultiLevelState) -> str:
        """Write state snapshot to file"""
        state_dir = self.target_dir / "state"
        state_dir.mkdir(parents=True, exist_ok=True)
        
        snapshot_path = state_dir / "multi-level-state.json"
        snapshot_path.write_text(json.dumps(state.to_dict(), indent=2))
        
        return str(snapshot_path)



"""
LLM Manager

Multi-provider LLM management with fallback support.

Mirrors: src/core/contract-ai/llm/llm-manager.ts
"""

import os
from pathlib import Path
from typing import Optional

from .provider import LLMProvider, LLMResponse, GenerateOptions, LLMProviderStatus, LLMModelInfo
from .ollama import OllamaClient, OllamaConfig
from .openrouter import OpenRouterClient
from .windsurf import WindsurfClient, WindsurfConfig


class ProviderInfo:
    """Information about a configured provider"""
    def __init__(self, name: str, status: LLMProviderStatus, provider: Optional[LLMProvider] = None):
        self.name = name
        self.status = status
        self.provider = provider
        self.models: list[LLMModelInfo] = []
        self.error: Optional[str] = None


class LLMManager:
    """
    LLM Manager with multi-provider support.
    
    Manages multiple LLM providers and provides fallback logic.
    
    Example:
        manager = LLMManager()
        await manager.initialize()
        
        if manager.is_available:
            response = await manager.generate(GenerateOptions(
                system="You are a code generator",
                user="Create a function to add two numbers"
            ))
            print(response.content)
    """
    
    def __init__(self, verbose: bool = False):
        self._providers: dict[str, LLMProvider] = {}
        self._provider_info: dict[str, ProviderInfo] = {}
        self._primary_provider: Optional[str] = None
        self._initialized = False
        self._verbose = verbose
    
    @property
    def is_available(self) -> bool:
        """Check if any provider is available"""
        return any(
            info.status == LLMProviderStatus.AVAILABLE 
            for info in self._provider_info.values()
        )
    
    def is_ready(self) -> bool:
        """Check if manager is initialized and has available provider"""
        return self._initialized and self.is_available
    
    def get_provider(self) -> Optional[LLMProvider]:
        """Get the primary available provider"""
        return self.primary_provider
    
    @property
    def primary_provider(self) -> Optional[LLMProvider]:
        """Get the primary (first available) provider"""
        if self._primary_provider and self._primary_provider in self._providers:
            return self._providers[self._primary_provider]
        return None
    
    @property
    def providers(self) -> dict[str, ProviderInfo]:
        """Get all provider info"""
        return self._provider_info
    
    async def initialize(self) -> None:
        """Initialize all configured providers"""
        if self._initialized:
            return

        self._load_env_file()
        
        # Initialize Ollama
        await self._init_ollama()
        
        # Initialize OpenRouter if configured
        await self._init_openrouter()
        
        # Initialize Windsurf
        await self._init_windsurf()

        preferred = os.getenv("LLM_PROVIDER", "auto").strip().lower() or "auto"

        if preferred not in ("", "auto"):
            info = self._provider_info.get(preferred)
            if info is not None and info.status == LLMProviderStatus.AVAILABLE:
                self._primary_provider = preferred

        if not self._primary_provider:
            if preferred == "auto":
                info = self._provider_info.get("openrouter")
                if info is not None and info.status == LLMProviderStatus.AVAILABLE:
                    self._primary_provider = "openrouter"

        if not self._primary_provider:
            for name, info in self._provider_info.items():
                if info.status == LLMProviderStatus.AVAILABLE:
                    self._primary_provider = name
                    break
        
        self._initialized = True

    def _load_env_file(self) -> None:
        candidates = [
            Path.cwd() / ".env",
            Path(__file__).resolve().parents[4] / ".env",
            Path.home() / ".reclapp" / ".env",
        ]

        for path in candidates:
            try:
                if not path.exists():
                    continue

                for line in path.read_text().splitlines():
                    line = line.strip()
                    if not line or line.startswith("#") or "=" not in line:
                        continue

                    key, value = line.split("=", 1)
                    key = key.strip()
                    value = value.strip().strip('"').strip("'")
                    if key and value and not os.environ.get(key):
                        os.environ[key] = value

                return
            except Exception:
                continue
    
    async def _init_ollama(self) -> None:
        """Initialize Ollama provider"""
        ollama = OllamaClient()
        info = ProviderInfo("ollama", LLMProviderStatus.UNAVAILABLE)
        
        try:
            if await ollama.is_available():
                models = await ollama.list_models()
                info.status = LLMProviderStatus.AVAILABLE
                info.provider = ollama
                info.models = models
                self._providers["ollama"] = ollama
            else:
                info.error = "Ollama not running"
        except Exception as e:
            info.status = LLMProviderStatus.ERROR
            info.error = str(e)
        
        self._provider_info["ollama"] = info
    
    async def _init_openrouter(self) -> None:
        """Initialize OpenRouter provider if API key is set"""
        api_key = os.getenv("OPENROUTER_API_KEY")

        info = ProviderInfo("openrouter", LLMProviderStatus.NOT_CONFIGURED)

        if not api_key:
            info.error = "Set OPENROUTER_API_KEY"
            self._provider_info["openrouter"] = info
            return

        try:
            client = OpenRouterClient()
            if await client.is_available():
                info.status = LLMProviderStatus.AVAILABLE
                info.provider = client
                self._providers["openrouter"] = client
            else:
                info.status = LLMProviderStatus.UNAVAILABLE
                info.error = "OpenRouter API key not configured"
        except Exception as e:
            info.status = LLMProviderStatus.ERROR
            info.error = str(e)

        self._provider_info["openrouter"] = info
    
    async def _init_windsurf(self) -> None:
        """Initialize Windsurf provider"""
        client = WindsurfClient()
        info = ProviderInfo("windsurf", LLMProviderStatus.UNAVAILABLE)
        
        try:
            if await client.is_available():
                models = await client.list_models()
                info.status = LLMProviderStatus.AVAILABLE
                info.provider = client
                info.models = models
                self._providers["windsurf"] = client
            else:
                info.error = "Windsurf server not running"
        except Exception as e:
            info.status = LLMProviderStatus.ERROR
            info.error = str(e)
        
        self._provider_info["windsurf"] = info

    def add_provider(self, name: str, provider: LLMProvider) -> None:
        """Add a custom provider"""
        self._providers[name] = provider
        self._provider_info[name] = ProviderInfo(
            name, 
            LLMProviderStatus.AVAILABLE,
            provider
        )
    
    async def generate(
        self, 
        options: GenerateOptions,
        provider_name: Optional[str] = None
    ) -> LLMResponse:
        """
        Generate completion using available provider.
        
        Args:
            options: Generation options
            provider_name: Specific provider to use (optional)
            
        Returns:
            LLM response
            
        Raises:
            RuntimeError: If no provider is available
        """
        if not self._initialized:
            await self.initialize()
        
        # Use specific provider if requested
        if provider_name:
            if provider_name not in self._providers:
                raise RuntimeError(f"Provider '{provider_name}' not available")
            return await self._providers[provider_name].generate(options)
        
        # Use primary provider
        if self.primary_provider:
            return await self.primary_provider.generate(options)
        
        raise RuntimeError("No LLM provider available. Run: reclapp setup")
    
    async def generate_with_fallback(
        self, 
        options: GenerateOptions,
        providers: Optional[list[str]] = None
    ) -> LLMResponse:
        """
        Generate with fallback to other providers on failure.
        
        Args:
            options: Generation options
            providers: List of providers to try (in order)
            
        Returns:
            LLM response from first successful provider
        """
        if not self._initialized:
            await self.initialize()
        
        provider_list = providers or list(self._providers.keys())
        last_error: Optional[Exception] = None
        
        for name in provider_list:
            if name not in self._providers:
                continue
            
            try:
                return await self._providers[name].generate(options)
            except Exception as e:
                last_error = e
                continue
        
        raise RuntimeError(f"All providers failed. Last error: {last_error}")
    
    def get_status(self) -> dict:
        """Get status of all providers"""
        return {
            name: {
                "status": info.status.value,
                "models": len(info.models),
                "code_models": len([m for m in info.models if m.is_code_model]),
                "error": info.error,
            }
            for name, info in self._provider_info.items()
        }
    
    async def close(self) -> None:
        """Close all provider connections"""
        for provider in self._providers.values():
            if hasattr(provider, 'close'):
                await provider.close()
    
    async def __aenter__(self):
        await self.initialize()
        return self
    
    async def __aexit__(self, *args):
        await self.close()



"""
Windsurf LLM Provider - Integration with Windsurf/Codeium AI

Windsurf uses a local server that proxies to various LLM providers.
Default endpoint: http://localhost:9090/v1/chat/completions

@version 1.0.0
"""

import os
import time
from typing import Optional

import httpx
from pydantic import BaseModel

from .provider import GenerateOptions, LLMModelInfo, LLMProvider, LLMResponse


class WindsurfConfig(BaseModel):
    """Configuration for Windsurf client"""
    base_url: str = "http://localhost:9090/v1"
    model: str = "cascade"  # Windsurf's default model
    timeout: int = 180
    api_key: str = ""  # Optional, some setups require it


class WindsurfClient(LLMProvider):
    """
    Windsurf/Codeium LLM provider.
    
    Windsurf provides a local proxy server that handles authentication
    and routes requests to various LLM providers (Claude, GPT-4, etc.)
    
    Environment variables:
    - WINDSURF_BASE_URL: Server URL (default: http://localhost:9090/v1)
    - WINDSURF_MODEL: Model to use (default: cascade)
    - WINDSURF_API_KEY: Optional API key
    
    Example:
        client = WindsurfClient()
        if await client.is_available():
            response = await client.generate(GenerateOptions(
                system="You are a helpful assistant",
                user="Hello!"
            ))
            print(response.content)
    """
    
    def __init__(self, config: Optional[WindsurfConfig] = None):
        if config is None:
            config = WindsurfConfig(
                base_url=os.getenv("WINDSURF_BASE_URL", "http://localhost:9090/v1"),
                model=os.getenv("WINDSURF_MODEL", "cascade"),
                api_key=os.getenv("WINDSURF_API_KEY", ""),
            )
        self.config = config
        self._client = httpx.AsyncClient(timeout=self.config.timeout)
        self._available: Optional[bool] = None
    
    @property
    def name(self) -> str:
        return "windsurf"
    
    @property
    def model(self) -> str:
        return self.config.model
    
    async def is_available(self) -> bool:
        """Check if Windsurf server is running and accessible"""
        if self._available is not None:
            return self._available
        
        try:
            url = self.config.base_url.rstrip("/") + "/models"
            headers = self._get_headers()
            resp = await self._client.get(url, headers=headers, timeout=5)
            self._available = resp.status_code == 200
        except Exception:
            self._available = False
        
        return self._available
    
    async def list_models(self) -> list[LLMModelInfo]:
        """List available models from Windsurf"""
        try:
            url = self.config.base_url.rstrip("/") + "/models"
            headers = self._get_headers()
            resp = await self._client.get(url, headers=headers)
            
            if resp.status_code != 200:
                return []
            
            data = resp.json()
            models = []
            
            for model in data.get("data", []):
                models.append(LLMModelInfo(
                    name=model.get("id", "unknown"),
                    size=None,
                    modified=None,
                    is_code_model=True  # Windsurf is primarily for code
                ))
            
            return models
        except Exception:
            return []
    
    async def has_model(self, model_name: Optional[str] = None) -> bool:
        """Check if a specific model is available"""
        if model_name is None:
            model_name = self.config.model
        
        models = await self.list_models()
        return any(m.name == model_name for m in models)
    
    async def generate(self, options: GenerateOptions) -> LLMResponse:
        """Generate completion using Windsurf"""
        if not await self.is_available():
            raise RuntimeError(
                "Windsurf server not available. "
                "Make sure Windsurf IDE is running or start the server manually."
            )
        
        start_time = time.time()
        
        headers = self._get_headers()
        
        payload: dict = {
            "model": self.config.model,
            "messages": [
                {"role": "system", "content": options.system},
                {"role": "user", "content": options.user},
            ],
            "temperature": options.temperature,
            "max_tokens": options.max_tokens,
        }
        
        if options.response_format == "json":
            payload["response_format"] = {"type": "json_object"}
        
        if options.stop:
            payload["stop"] = options.stop
        
        url = self.config.base_url.rstrip("/") + "/chat/completions"
        
        try:
            resp = await self._client.post(url, headers=headers, json=payload)
            resp.raise_for_status()
            data = resp.json()
        except httpx.HTTPStatusError as e:
            raise RuntimeError(f"Windsurf API error: {e.response.status_code} - {e.response.text}")
        except Exception as e:
            raise RuntimeError(f"Windsurf request failed: {e}")
        
        # Parse response
        content = ""
        if "choices" in data and data["choices"]:
            message = data["choices"][0].get("message", {})
            content = message.get("content", "")
        
        usage = data.get("usage", {})
        tokens_used = usage.get("total_tokens", 0)
        duration_ms = int((time.time() - start_time) * 1000)
        
        return LLMResponse(
            content=content,
            model=self.config.model,
            provider="windsurf",
            tokens_used=tokens_used,
            duration_ms=duration_ms,
            raw=data,
        )
    
    def _get_headers(self) -> dict[str, str]:
        """Get request headers"""
        headers = {
            "Content-Type": "application/json",
        }
        
        if self.config.api_key:
            headers["Authorization"] = f"Bearer {self.config.api_key}"
        
        return headers
    
    async def close(self):
        """Close the HTTP client"""
        await self._client.aclose()
    
    async def __aenter__(self):
        return self
    
    async def __aexit__(self, *args):
        await self.close()
